/** \file pruio.h
\brief C header file for libpruio.

Header file for including libpruio to your program. It contains the
declarations.

*/


//! tell pruss_intc_mapping.bi that we use ARM33xx
#define AM33XX

//! the PRUSS driver library
#include "prussdrv.h"
//! PRUSS driver interrupt settings
#include "pruss_intc_mapping.h"

//! the channel for PRU messages (must match PRUIO_IRPT)
#define PRUIO_CHAN   CHANNEL5
//! the mask to enable PRU interrupts (must match PRUIO_IRPT)
#define PRUIO_MASK   PRU_EVTOUT5_HOSTEN_MASK
//! the event for PRU messages (mapping, must match PRUIO_IRPT)
#define PRUIO_EMAP   PRU_EVTOUT5
//! the event for PRU messages (must match PRUIO_IRPT)
#define PRUIO_EVNT   PRU_EVTOUT_5

//! the default setting for avaraging
#define PRUIO_DEF_AVRAGE   4
//! the default value for open delay in channel settings
#define PRUIO_DEF_ODELAY   183
//! the default value for sample delay in channel settings
#define PRUIO_DEF_SDELAY   0
//! the default PRU subsystem to use
#define PRUIO_DEF_PRUSS    1
//! the default number of samples to use (configures single mode)
#define PRUIO_DEF_SAMPLS   0
//! the default step mask (steps 1 to 8 for AIN0 to AIN7, no charge step)
#define PRUIO_DEF_STPMSK   0x1FE /* &b111111110 */
//! the default timer value (sampling rate)
#define PRUIO_DEF_TIMERV   0
//! the default bit mode (4 = 16 bit encoding)
#define PRUIO_DEF_LSLMOD   4
//! the default clock divisor (0 = full speed AFE = 2.4 MHz)
#define PRUIO_DEF_CLKDIV   0

//! convenience macro to configure an input pin without pullup or pulldown
#define PRUIO_IN    0x2F /*   &b00101111 */
//! convenience macro to configure an low input pin (with pulldown resistor)
#define PRUIO_IN_0  0x27 /*   &b00100111 */
//! convenience macro to configure an high input pin (with pullup resistor)
#define PRUIO_IN_1  0x37 /*   &b00110111 */
//! convenience macro to configure an output pin initial low
#define PRUIO_OUT0   0xF /*   &b00001111 */
//! convenience macro to configure an output pin initial high
#define PRUIO_OUT1  0x8F /*   &b10001111 */
//! convenience macro to re-configure a pin to its original (initial) state
#define PRUIO_PIN_RESET 0xFF /*  &b11111111 */

enum PRUIO_PIN_LOCKING{
  PRUIO_UNLOCK_CURR, //!< unlock a pin, don't change the configuration
  PRUIO_UNLOCK_ORIG, //!< unlock a pin, change to original (initial) configuration
  PRUIO_UNLOCK_NEW,  //!< unlock a pin, change to parameter configuration
  PRUIO_LOCK_CURR,   //!< lock a pin, don't change the configuration
  PRUIO_LOCK_ORIG,   //!< lock a pin, change to original (initial) configuration
  PRUIO_LOCK_NEW,    //!< lock a pin, change to parameter configuration
  PRUIO_LOCK_CHECK   //!< don't change pin locking, skip operation if pin is locked
};

typedef unsigned int uint32;   //!< 32 bit unsigned integer data type
typedef unsigned short uint16; //!< 16 bit unsigned integer data type
typedef unsigned char uint8;   //!< 8 bit unsigned integer data type
typedef int int32;              //!< 32 bit signed integer data type
typedef short int16;            //!< 16 bit signed integer data type
typedef signed char int8;      //!< 8 bit signed integer data type

//! user defined type for a step setting
struct AdcSteps{
  uint32 Confg, //!< context for configuration register
          Delay; //!< context for delay register
};

//! user defined type for a GPIO setting
struct GpioSet{
  uint32
    Addr,  //!< base address of GPIO device
    ClrM,  //!< mask to clear pins to low
    SetM,  //!< mask to set pins to high
    OutE,  //!< mask to enable pins for output
    Stat,  //!< current state of pins (mixed)
    StaO,  //!< current state of output pins
    fe1,   //!< future expansion
    fe2;
};

//! user defined type (class) for the driver structure
typedef struct PruIo{

  uint32
    Samples,   //!< number of samples
    TimerVal,  //!< timer value in [ns]
    StepMask;  //!< mask to specify active channels
  uint16
    LslMode,   //!< bit shift modus (0 to 4)
    ClockDiv;  //!< divisor for ADC input clock (register 0x4C)

  uint32
     AdcReg24   //!< ADC register IRQ_STATUS_RAW
    ,AdcReg28   //!< ADC register IRQ_STATUS
    ,AdcReg2C   //!< ADC register IRQENABLE_SET
    ,AdcReg30   //!< ADC register IRQENABLE_CLR
    ,AdcReg34   //!< ADC register IRQWAKEUP
    ,AdcReg38   //!< ADC register DMAENABLE_SET
    ,AdcReg3C   //!< ADC register DMAENABLE_CLR
    ,AdcReg40;  //!< ADC register CRTL (bits 0, 1 and 2 gets set by driver)

//! step configuration (charge step + 16 steps + idle step, by default steps 1 to 8 are used for AIN0 to AIN7)
  struct AdcSteps St_p[17 + 1];

  uint32* DRam;//!< pointer to access PRU DRam


  void *ERam,      //!< pointer to read PRU external ram
     *BallOrg;      //!< buffer pointer for original Ball configuration
  uint32* AdcOrg,  //!< buffer pointer for original ADC configuration (= 0 in case of ADC down at startup)
         *GpioOrg;  //!< buffer pointer for original GPIO configuration
  uint16* Value;   //!< pointer to read sampled ADC input data
  uint8* BallRef,  //!< pointer to ball reference (GPIO# for CPU pins or lock)
        *BallConf;  //!< pointer to ball configuration (CPU pin muxing)
  char* Errr;      //!< pointer for error messages
  uint32
    EOffs,      //!< the offset of the external PRUSS-DDR
    ESize,      //!< the size of the external PRUSS-DDR
    CSize,      //!< the size of the original configuration block
    PruNo,      //!< the PRU number to use
    PruEvtout,  //!< the interrupt channel to send commands to PRU
    PruIRam,    //!< the PRU instruction ram to load
    PruDRam;    //!< the PRU data ram
  int16
    ArmPruInt,  //!< the interrupt to send
    ChAz;       //!< number of active channels

//! interrupt settings (we also set default interrupts, so that the other PRUSS can be used in parallel)
  struct __pruss_intc_initdata IntcInit; /* = {
    { PRU0_PRU1_INTERRUPT
    , PRU1_PRU0_INTERRUPT
    , PRU0_ARM_INTERRUPT
    , PRU1_ARM_INTERRUPT
    , ARM_PRU0_INTERRUPT
    , ARM_PRU1_INTERRUPT
    , PRUIO_IRPT
    ,  (char)-1  },
    { {PRU0_PRU1_INTERRUPT,CHANNEL1}
    , {PRU1_PRU0_INTERRUPT, CHANNEL0}
    , {PRU0_ARM_INTERRUPT,CHANNEL2}
    , {PRU1_ARM_INTERRUPT, CHANNEL3}
    , {ARM_PRU0_INTERRUPT, CHANNEL0}
    , {ARM_PRU1_INTERRUPT, CHANNEL1}
    , {PRUIO_IRPT, PRUIO_CHAN}
    ,  {-1,-1}},
     {  {CHANNEL0,PRU0}
      , {CHANNEL1, PRU1}
      , {CHANNEL2, PRU_EVTOUT0}
      , {CHANNEL3, PRU_EVTOUT1}
      , {PRUIO_CHAN, PRUIO_EMAP}
      , {-1,-1} },
      (PRU0_HOSTEN_MASK | PRU1_HOSTEN_MASK | PRU_EVTOUT0_HOSTEN_MASK | PRU_EVTOUT1_HOSTEN_MASK | PRUIO_MASK)
  } */

  struct GpioSet* Gpio;//!< pointer to GPIO structure (current values)
//! initial GPIO definitions
  //struct GpioSet GpioInit[PRUIO_GPIO_AZ + 1]; /* = {
  struct GpioSet GpioInit[3 + 1]; /* = {
    TYPE<GpioSet>(&h44E07100uL, &h44E00408uL, 0, 0, 0, 0, 0, 0)
  , TYPE<GpioSet>(&h4804C100uL, &h44E000ACuL, 0, 0, 0, 0, 0, 0)
  , TYPE<GpioSet>(&h481AC100uL, &h44E000B0uL, 0, 0, 0, 0, 0, 0)
  , TYPE<GpioSet>(&h481AE100uL, &h44E000B4uL, 0, 0, 0, 0, 0, 0)
    } */

//! list of GPIO numbers, corresponding to ball index
  //uint8 BallGpio[PRUIO_BALL_AZ + 1]; /* = {
  uint8 BallGpio[109 + 1]; /* = {
     32,  33,  34,  35,  36,   37,  38,  39,  22,  23
  ,  26,  27,  44,  45,  46,   47,  48,  49,  50,  51
  ,  52,  53,  54,  55,  56,   57,  58,  59,  30,  31
  ,  60,  61,  62,  63,  64,   65,  66,  67,  68,  69
  ,  70,  71,  72,  73,  74,   75,  76,  77,  78,  79
  ,  80,  81,   8,   9,  10,   11,  86,  87,  88,  89
  ,  90,  91,  92,  93,  94,   95,  96,  97,  98,  99
  , 100,  16,  17,  21,  28,  105, 106,  82,  83,  84
  ,  85,  29,   0,   1,   2,    3,   4,   5,   6,   7
  ,  40,  41,  42,  43,  12,   13,  14,  15, 101, 102
  , 110, 111, 112, 113, 114,  115, 116, 117,  19,  20} */
} PruIo;

