/** \file pruio_c_wrapper.h
\brief The main header code of the C wrapper for libpruio.

This file provides the declarations of the \ref PruIo class member
functions in C syntax. It also can be used to create language bindings
for non-C languages to use libpruio in polyglot applications.

\note The includes pruio.h and pruio_pins.h are not shown in this
      documentation because they confuse Doxygen. It mixes up
      references with the original headers pruio/pruio.bi and
      pruio/pruio_pins.bi.

Licence: LGPLv2

Copyright 2014 by Thomas{ dOt ]Freiherr[ At ]gmx[ DoT }net

*/

#include "pruio.h"
#include "pruio.hp"

/** \brief C wrapper to create a \ref PruIo instance.
\param Av avaraging for default steps (0 to 16, defaults to 0)
\param OpD open delay for default steps (0 to 0x3FFFF, defaults to 0x98)
\param SaD sample delay for default steps (0 to 255, defaults to 0)
\param Pru number of PRU subsystem to use (0 or 1, defaults to 1)
\returns A pointer for the new instance.

This function wraps the constructor, check \ref PruIo::PruIo() for
details.

Since the constructor reads the original devices configurations and the
destructor restores them, it's recommended to create and use just one
\ref PruIo instance at the same time.

*/
PruIo* pruio_new(uint8 Av, uint32 OpD, uint8 SaD, uint8 Pru);

/** \brief C wrapper to destroy a \ref PruIo instance.
\param Io The pointer of the instance.

This function wraps the destructor, check \ref PruIo::~PruIo() for
details. (Doxygen should create a link here?!)

*/
void pruio_destroy(PruIo* Io);

/** \brief C wrapper for function \ref PruIo::config().
\param Io The pointer of the \ref PruIo instance
\param Samp number of samples to fetch (defaults to zero)
\param Mask mask for active steps (defaults to all 8 channels active in steps 1 to 8)
\param Tmr timer value in [ns] to specify the sampling rate (defaults to zero, MM only)
\param Mds modus for output (defaults to 4 = 16 bit)
\param ClDv divisor for ADC input clock (defaults to 0 = full speed = 2.4 MHz)
\returns zero on success (otherwise a string with an error message)

This function wraps the member function config, check \ref
PruIo::config() for details.

*/
char* pruio_config(PruIo* Io, uint32 Samp, uint32 Mask, uint32 Tmr, uint16 Mds, uint16 ClDv);

/** \brief C wrapper for function \ref PruIo::adc_step().
\param Io The pointer of the \ref PruIo instance
\param Stp step index (0 = step 0 => charge step, 1 = step 1 (=> AIN0 by default),  ..., 17 = idle step)
\param ChN channel number to scan (0 = AIN0, 1 = AIN1, ...)
\param Av new value for avaraging (defaults to 4)
\param SaD new value for sample delay (defaults to 0)
\param OpD new value for open delay (defaults to 0x98)
\returns zero on success (otherwise a string with an error message)

This function wraps the member function adc_step, check \ref
PruIo::adc_step() for details.

*/
char* pruio_adc_step(PruIo* Io, uint8 Stp, uint8 ChN, uint8 Av, uint8 SaD, uint32 OpD);

/** \brief C wrapper for function \ref PruIo::gpio_get_config().
\param Io The pointer of the \ref PruIo instance
\param Ball the CPU ball number to describe
\returns a human-readable text string (internal string, never free it)

This function wraps the member function adc_step, check \ref
PruIo::gpio_get_config() for details.

*/
char* pruio_gpio_get_config(PruIo* Io, uint8 Ball);

/** \brief C wrapper for function \ref PruIo::gpio_set().
\param Io The pointer of the \ref PruIo instance
\param Ball the CPU ball number to set
\param Modus the mode for the GPIO
\param Lokk the locking specification
\returns zero on success (otherwise a pionter to an error message)

This function wraps the member function adc_step, check \ref
PruIo::gpio_set() for details.

*/
char* pruio_gpio_set(PruIo* Io, uint8 Ball, uint8 Modus, uint8 Lokk);

/** \brief C wrapper for function \ref PruIo::gpio_get().
\param Io The pointer of the \ref PruIo instance
\param Ball the CPU ball number to test
\returns GPIO state (otherwise -1, check \ref PruIo::Errr for an error message)

This function wraps the member function adc_step, check \ref
PruIo::gpio_get() for details.

*/
uint32 pruio_gpio_get(PruIo* Io, uint8 Ball);

/** \brief C wrapper for function \ref PruIo::gpio_out().
\param Io The pointer of the \ref PruIo instance
\param Ball the CPU ball number to test
\param Modus the state to set (0 = low, high otherwise)
\returns zero on success (otherwise a pointer to an error message)

This function wraps the member function adc_step, check \ref
PruIo::gpio_out() for details.

*/
char* pruio_gpio_out(PruIo* Io, uint8 Ball, uint8 Modus);

/** \brief C wrapper for function \ref PruIo::mm_trg_pin().
\param Io The pointer of the \ref PruIo instance
\param Ball the CPU ball number to test
\param GpioV the state to check (defaults to high = 1)
\param Skip the number of samples to skip (defaults to 0 = zero, max. 1023)
\returns the trigger configuration (or zero in case of an error, check \ref PruIo::Errr)

This function wraps the member function adc_step, check \ref
PruIo::mm_trg_pin() for details.

*/
uint32 pruio_mm_trg_pin(PruIo* Io, uint8 Ball, uint8 GpioV, uint16 Skip);

/** \brief C wrapper for function \ref PruIo::mm_trg_ain().
\param Io The pointer of the \ref PruIo instance
\param Stp the step number to use for trigger input
\param AdcV the sample value to match (positive check greater than, negative check less than)
\param Rela if AdcV is relative to the current input
\param Skip the number of samples to skip (defaults to 0 = zero, max. 1023)
\returns the trigger configuration (or zero in case of an error, check \ref PruIo::Errr)

This function wraps the member function adc_step, check \ref
PruIo::mm_trg_ain() for details.

*/
uint32 pruio_mm_trg_ain(PruIo* Io, uint8 Stp, int32 AdcV, uint8 Rela, uint16 Skip);

/** \brief C wrapper for function \ref PruIo::mm_trg_pre().
\param Io The pointer of the \ref PruIo instance
\param Stp the step number to use for trigger input
\param AdcV the sample value to match (positive check greater than, negative check less than)
\param Samp the number of samples for the pre-trigger
\param Rela if AdcV is relative to the current input
\returns the trigger configuration (or zero in case of an error, check \ref PruIo::Errr)

This function wraps the member function adc_step, check \ref
PruIo::mm_trg_pre() for details.

*/
uint32 pruio_mm_trg_pre(PruIo* Io, uint8 Stp, int32 AdcV, uint16 Samp, uint8 Rela);

/** \brief C wrapper for function \ref PruIo::mm_start().
\param Io The pointer of the \ref PruIo instance
\param Trg1 settings for first trigger (default = no trigger)
\param Trg2 settings for second trigger (default = no trigger)
\param Trg3 settings for third trigger (default = no trigger)
\param Trg4 settings for fourth trigger (default = no trigger)
\returns zero on success (otherwise a string with an error message)

This function wraps the member function adc_step, check \ref
PruIo::mm_start() for details.

*/
char* pruio_mm_start(PruIo* Io, uint32 Trg1, uint32 Trg2, uint32 Trg3, uint32 Trg4);
